import copy
import numpy
import re
import netCDF4
import textwrap
import collections
#from .utils import equals, _set_tolerances, Comparison
import cf

class A(object):

    def __init__(self, *args):

        # Initialize self._list, bypassing setattr so that '_list'
        # does not end up in the self._properties set
        self._list = list(sequence)

    #--- End: def



class CfList(collections.MutableSequence):
    '''

    A list-like object (a 'cf list') suitable for CF structures.
 
    When requesting a non-private attribute that the cf list does not
    have, and the :attr:`_elements_atts` attribute is True, instead of
    raising an AttributeError a cf list is returned containing the
    values of the attribute from each of the elements, with a value of
    None if an element does not have the attribute. The setting of an
    attribute is as usual, i.e. it is set on the object itself rather
    that the object's elements.

    :argument sequence: *Optional*. Initialize new list from
                        sequence's items.

    :type sequence: iterable

    **Overloaded operators**

    The ``in`` (set membership), ``==`` and ``!=`` operators are
    overloaded to use numerically tolerant equality.

    The ``+, *, +=`` and ``*=`` operators behave as for a built-in
    list. Note in particular that in-place changes to an element of
    the result of one of these operations will affect the equivalent
    element on the right hand side, and possibly other equivalent
    elements in the result itself.
 
    ====================== ======================================================
    Attribute              (`type`) Description
    ====================== ======================================================
    :attr:`_elements_atts` (:class:`bool`) If True then do not raise
                           an exception when requesting a non-existent
                           attribute, but broadcast the request to
                           each element of the list.
    ====================== ======================================================

    ============================= ===============================================================
    Method                        Description
    ============================= ===============================================================
    :meth:`~cf.CfList.append`     As for a built-in list.
    :meth:`~cf.CfList.copy`       Create a deep copy.
    :meth:`~cf.CfList.count`      As for a built-in list but using numerically tolerant equality.
    :meth:`~cf.CfList.equals`     Determine whether two lists are congruent element-wise.
    :meth:`~cf.CfList.extend`     As for a built-in list.
    :meth:`~cf.CfList.index`      As for a built-in list but using numerically tolerant equality.
    :meth:`~cf.CfList.insert`     As for a built-in list.
    :meth:`~cf.CfList.pop`        As for a built-in list.
    :meth:`~cf.CfList.properties` Return a set of writable, public attributes.
    :meth:`~cf.CfList.remove`     As for a built-in list.
    :meth:`~cf.CfList.reverse`    As for a built-in list.
    :meth:`~cf.CfList.sort`       As for a built-in list.
    ============================= ===============================================================

    '''

    def __init__(self, sequence=()):

        # return elements' attributes if not set an attribute is not
        # defined on self
        self._elements_atts = True

        # Initialize self._list, bypassing setattr so that '_list'
        # does not end up in the self._properties set
        self._list = list(sequence)

        # Note: the extend works because s.extend(x) is the same as
        # s[len(s):len(s)]=x and we have defined __setitem__ to work
        # on s._list

    #--- End: def

    def __repr__(self):
        '''x.__repr__() <==> repr(x)'''
        return repr(self._list)

    def __setattr__(self, attr, value):
                
        # See Variable.__setattr__ for notes.
        if not attr.startswith('_'):
            try:
                self.__dict__['_properties'].add(attr)
            except KeyError:
                self.__dict__['_properties'] = set((attr,))
        super(CfList, self).__setattr__(attr, value)
    #--- End: def CfList.__setattr__

#        if attr.startswith('__'):        
# #           super(CfList, self).__setattr__(attr, value)
 # #      else:
  #      try:
   #         super(CfList, self).__getattr__(attr)
#    ##    except AttributeError:
#            for variable in self:
#                if isinstance(variable, cf.CfList):
#                    raise RuntimeError('asdasdasdas')
#                print  repr(variable)
#                variable.__setattr__(attr, value)
#        else:
#            raise AttributeError('read only')
#    #--- End: def
            

 #       # See Variable.__setattr__ for notes.
#
     #       try:
    #            self.__dict__['_properties'].add(attr)
   #         except KeyError:
  #              self.__dict__['_properties'] = set((attr,))
 #       super(CfList, self).__setattr__(attr, value)
#    #--- End: def CfList.__setattr__

    def __delattr__(self, attr):

        # Delete an attribute and remove it from the _properties set
        try:
            self.__dict__['_properties'].remove(attr)
        except (AttributeError, KeyError):
            pass
        super(CfList, self).__delattr__(attr)
    #--- End: def

#    def _get_element_attr(self, x, attr):
 #       try:
  #          return getattr(x, attr)
   #     except AttributeError:
    #        return None
    #--- End: def

    def __getattribute__(self, attr):
        '''

    When requesting an attribute that the instance does not have, and
    the :attr:`_elements_atts` attribute is True, instead of raising
    an AttributeError a cf list is returned of the values of the
    attribute from each of the elements, with a value of None if an
    element does not have the attribute.
   
    :returns: A built-in list.

    ''' 
        try:
            # Return an attribute of the list object
            return super(CfList, self).__getattribute__(attr)
        except AttributeError:
            if attr.startswith('__'):
                return super(CfList, self).__getattribute__(attr)

            # Return a built-in list of the attributes from each
            # element of the cf list
            attrs = CfList()
            for variable in self:
                try:
                    attrs.append(getattr(variable, attr))
                except AttributeError:
                    attrs.append(None) 
            return attrs

    def __contains__(self, item):
        '''

        Test for set membership using numerically tolerant equality.

        '''
        for s in self._list:
            if cf.equals(item, s):
                return True
                
        return False
    #--- End: def

    def __len__(self):
        return len(self._list)

    def __delitem__(self, index):
        del self._list[index]

    def __getitem__(self, index):
        '''

    Implement indexing of the cf list.

    :returns: A cf list which is a shallow copy of a subset of the
              original cf list.

    '''    
#        subset = self._list[index]
        subset = (super(CfList, self).__getattribute__('_list'))[index]
        if isinstance(subset, list):
            return type(self)(subset)
        else:
            return subset
    #--- End: def

    def __setitem__(self, index, value):
        self._list[index] = value

    def __copy__(self):
        '''

    Used if copy.copy is called on the list.
    
    '''     
        return self[:]
    #--- End: def CfList.__copy__

    def __deepcopy__(self, memo):
        '''

    Used if copy.deepcopy is called on the list.
    
    .. seealso:: The :meth:`~cf.CfList.copy` method.

    ''' 
        return self.copy()
    #--- End: def CfList.__deepcopy__

    def __eq__(self, other):
        '''

    Return True if two instances are congruent in that each pair of
    their elements are equal.
            
    Numerical equality is to within a tolerance. Refer to the
    :meth:`~cf.CfList.equals` method for details.

    '''
        return self.equals(other)
    #--- End: def CfList.__eq__

    def __ne__(self, other):
        '''

    Return True if two instances are not congruent in that each pair
    of their elements are not equal.
            
    Numerical equality is to within a tolerance. Refer to the
    :meth:`~cf.CfList.equals` method for details.

    '''

        return not self.equals(other)
    #--- End: def CfList.__ne__    

    def __add__(self, other):
        '''
     x.__add__(y) <==> x+y

     Implement the + binary operation make to concatenate two lists.
     
     :returns: A cf list.
 
        '''
        new = copy.deepcopy(self)
        new.extend(other)
        return new
    #--- End def CfList.__add__

    def __mul__(self, other):
        '''

     Implement the * binary operation make concatenated shallow
     copies of the list.
     
     The right hand side of the operation must be an integer.

     :returns: A cf list.
        '''
        if not isinstance(other, int):
            raise TypeError("can't multiply cf list by non-int of type '"+ \
                                other.__class__.__name__+"'")
        
        new = type(self)()
        
        if other <= 0:
            # Return empty list for non-positive integer
            return new
        
        for i in xrange(other):
            new.extend(self)
            
        return new
    #--- End def CfList.__mul__

    def __rmul__(self, other):
        '''

     Implement the * binary operation make concatenated shallow
     copies of the list.
     
     The left hand side of the operation must be an integer.

     :returns: A cf list.
     '''
        return self.__mul__(other)
    #--- End def CfList.__rmul__

    def __iadd__(self, other):
        '''

     Implement the += augmented assignment make to extend the list in
     place.
     
     :returns: A cf list.
        '''
        self.extend(other[:])
        return self
    #--- End def CfList.__iadd__
        
    def __imul__(self, other):
        '''

     Implement the *= augmented assignment make concatenated shallow
     copies of the list in place.
     
     The right hand side of the operation must be an integer.

     :returns: A cf list.
     '''
        self = self.__mul__(other)
        return self
    #--- End def CfList.__imul__

    def count(self, value):
        '''

    :returns: The number of occurrences of value using numerically
              tolerant equality.

    '''
        rtol = cf.RTOL
        atol = cf.ATOL

        return sum(1 for v in self if cf.equals(value, v, 
                                                rtol=rtol, atol=atol))
    #--- End def CfList.count

    def index(self, value, start=0, stop=None):
        '''

    :returns: The first index of value using numerically tolerant
              equality. Restrict the search to the slice
              :obj:`start`::obj:`stop`. If :obj:`stop` is None then
              the slice is :obj:`start:`cf.

              '''
        rtol = self.RTOL
        atol = self.ATOL

        if start < 0:
            start = len(self) + start
        if stop is None:
            stop = len(self)
        elif stop < 0:
            stop = len(self) + stop

	for i, v in enumerate(self):  
            if i < start or i >= stop:
                continue
            if cf.equals(value, v, rtol=rtol, atol=atol):
                return i
	raise ValueError(self.__class__.__name__+'.index(x): x not in list')
    #--- End: def

    def insert(self, index, object):
        '''

    L.insert(index, object) -- insert object before index

    '''
        self._list.insert(index, object)
    #--- End: def CfList.insert

    def copy(self):
        '''

    Create a deep copy of the list. Equivalent to calling
    copy.deepcopy on the list.

    :returns: A deep copy of the list.

    '''
        new = type(self)()
    
        for x in self:
            new.append(copy.deepcopy(x))

        return new
    #--- End: def CfList.copy

    def equals(self, other, rtol=None, atol=None):
        '''

Return True if two instances are congruent in that each pair of their
elements are equal.
        
Numerical equality is to within a tolerance. Refer to :mod:`cf` for
details.

:argument other: The variable to compare against for equality.

:type other: object
                                                 
:keyword atol: *Optional*. If None then use the default method for
               setting the absolute tolerance for numerical equality
               (refer to :mod:`cf` for details). If a number then set
               the absolute tolerance to this value for all such
               comparisons.

:type atol: None or float

:keyword rtol: *Optional*. If None then use the default method for
               setting the relative tolerance for numerical equality
               (refer to :mod:`cf` for details). If a number then set
               the relative tolerance to this value for all such
               comparisons.

:type rtol: None or float


:returns: True if the two objects are congruent, False otherwise.

    '''
        if id(self) == id(other):
            return True
         
        # Check that each instance is the same type
        if type(self) != type(other):
            return False

        # Check that the lists have the same number of elements
        if len(self) != len(other):
            return False
          
        # Check properties
        self_properties = self.properties()
        if self_properties != other.properties():
            return False
               
        if rtol is None:
            rtol = cf.RTOL
        if atol is None:
            atol = cf.ATOL                
     
        for prop in self_properties: 
            x = getattr(self, prop)
            y = getattr(other, prop)
            if not cf.equals(x, y, rtol=rtol, atol=atol):
                return False

        # Check the element values
        for i in xrange(len(self)):
            if not cf.equals(self[i], other[i], rtol=rtol, atol=atol):
                return False

        return True
    #--- End: def CfList.equals

    def properties(self, nc=False):
        # __doc__ is copied from _find_properties.__doc__
        
        return _find_properties(self, nc=nc)
    #--- End: def CfList.properties
    properties.__doc__ = _find_properties.__doc__
    
#--- End: class CfList            
