def agg(fields, unaggregatable):

#- Both fields have the same standard_name, etc.

#- Each dimension in each space has at least one scalar or 1d
#  coordinate (dimension or auxiliary) and each such coordinate's
#  standard_name is unique within its space.

    if self.ndim => other.ndim:
        field0 = self
        field1 = other
    else:
        field0 = other
        field1 = self

    space0 = field0.space
    space1 = field1.space

    # Raise an exception for missing functionality
    for attr in ('ancillary_variables', 'flag_values', 'flag_masks', 'flag_meanings'):
        for field in (field0, field1):
            try:
                getattr(field, attr)
                raise NotImplementedError, "Haven't sorted out %(attr)s, yet." % locals()
            except AttributeError:
                pass
    #--- End: for

    # Test standard name
    try:
        standared_name0 = field0.standard_name
    except AttributeError:
        # Stop: Field has no standard name. UNAGGREGATABLE
        unaggregatable.append(field0)
        return False
    try:
        standared_name1 = field0.standard_name
    except AttributeError:
        # Stop: Fields have different standard names
        return False
    if not equals(field0.standard_name, field1.standard_name):
        # Stop: Fields have different standard names
        return False

    # Test units. Note that this is currently too strict - non-equal
    # but equivalent units are currently not allowed
    try:
        if not equals(field0.units, field1.units):
            # Stop: Fields have different units
            return False
    except AttributeError:
        # Stop: At least one field does not have units. UNAGGREGATABLE
        return False

    if len(space0.dimension_sizes) != len(space1.dimension_sizes):
        # Stop: Fields' spaces have different ranks
        return False
         
    cfatts0 = set(field0.cfatts) - set(('history', 'cell_methods', 'standard_name', 'units'))
    cfatts1 = set(field1.cfatts) - set(('history', 'cell_methods', 'standard_name', 'units'))

    if cfatts0 != cfatts1:
        # Stop: Fields have different sets of attributes
        return False
    for attr in cfatts0:
        if not equals(getattr(field0, attr), getattr(field1, attr)):
            # Stop: Fields have different values for an attribute
            return False

    # Each dimension in each space has at least one scalar or 1d
    # coordinate (dimension or auxiliary) and each such coordinate's
    # standard_name is unique within its space.
    meta0 = CfDict()
    meta1 = CfDict()

    meta  = meta0
    space = space0
    field = field0
    for n in (0, 1):

        all_standard_names     = []
        meta.nd_coords         = []
        meta.nd_standard_names = []

        for key in space:
            coord = space[key]            
            
            try:
                standard_name = coord.standard_name
            except AttributeError:
                # Stop: Coordinate or cell measure does not have a
                #       standard_name. UNAGGREGATABLE
                unaggregatable.append(field)
                return False
            all_standard_names.append(standard_name)

            try:
                if coord.ndim <= 1:
                    dim = space.dimensions[key][0]                    
                else:
                    meta.nd_coords.append(key)
                    meta.nd_standard_names.append(standard_name)
                    continue
            except AttributeError:
                # Stop: Coordinate or cell measure has no
                #       data. UNAGGREGATABLE
                unaggregatable.append(field)
                return False

            # Still here?
            try:
                meta[dim]['1d_coords'].append(key)
                meta[dim]['1d_standard_names'].append(standard_name)
            except KeyError:
                meta[dim] = {'1d_coords'        : [key],
                             '1d_standard_names': [standard_name]}
        #--- End: for

        if len(all_standard_names) > len(set(all_standard_names)):
            # Stop: Two or more coordinates/cell measures in the same
            #       field share the same standard name. UNAGGREGATABLE
            unaggregatable.append(field)
            return False

        # Check that every dimension of the space has at least one
        # dimension or auxiliary coordinate    
        for dim in space.dimension_sizes:
            try:
                meta[dim]
            except KeyError:
                # Stop: No 1d dimension or auxiliary coordinates for
                #       this dimension. UNAGGREGATABLE
                unaggregatable.append(field)
                return False

        # Create a set of each dimension's 1d coordinates' standard
        # names
        meta[dim]['set_1d_standard_names'] = \
            set(meta[dim]['1d_standard_names'])

        # Change space ready for the next loop iteration
        meta  = meta1
        space = space1
        field = field1
    #--- End: for

    # ----------------------------------------------------------------
    # Check the cell methods. Do this after ascertaining that every
    # coordinate has a standard name so we can be sure that the 'name'
    # keys are comparable
    # ----------------------------------------------------------------
    try:
        cell_method0 = field0.cell_methods
    except AttributeError:
        try:
            field1.cell_methods
            # Stop: Only one of the fields has cell methods
            return False
        except AttributeError:
            # Neither field has cell methods
            pass
    else:
        try:
            cell_method1 = field1.cell_methods
        except AttributeError:
            # Stop: Only one of the fields has cell methods
            return False

        # ------------------------------------------------------------
        # Both fields have cell methods
        # ------------------------------------------------------------
        if len(cell_method0) != len(cell_method1):
            # Stop: Cell methods of different lengths
            return False

        for cm0, cm1 in zip(cell_method0, cell_method1):
            keys0 = set(cm0) - set(('dim',))
            keys1 = set(cm1) - set(('dim',))
        
            if keys0 != keys1:
                # Stop: Different cell method
                return False

            for key in keys0:
                if not equals(cm0[key], cm1[key]):
                    # Stop: Different cell method
                    return False
        #--- End: for
    #--- End: try

    # ================================================================
    # For each dimension in one of the spaces, there is a 'matching
    # dimension' in the other space with exactly the same list of
    # scalar or 1d coordinate (dimension or auxiliary) standard_names,
    # although a standard_name may belong to different types of
    # coordinate in the two spaces. Call these coordinates a pair of
    # 'matching coordinates' for the matching dimensions
    # ================================================================
    matching_dimensions = {}
    dims1 = space1.dimension_sizes.keys()

    for dim0 in space0.dimension_sizes:
        # Make sure name 'dim1' is not defined
        try:
            del dim1
        except NameError:
            pass

        standard_names0 = meta0[dim0]['1d_standard_names'][0]
        for i in xrange(len(dims1)):
            dim = dims1[i]
            if standard_names0 in meta1[dim]['set_1d_standard_names']:
                # Found a potentially matching dimension
                dim1 = dim
                dims1.pop(i)
                break
        #--- End: for

        try:
            matched_dims = (dim0, dim1)
        except NameError:
            # Stop: This dimension in field0 has no matching dimension
            #       in field1 (because name 'dim1' is not defined)
            return False

        # Now we have a pair of potentially matching dimensions, check
        # to see if it really is; and if it is then record its
        # matching coordinates.
        matching_dimensions[matched_dims] = []
        for key0, sn0 in zip(meta0[dim0]['1d_coord'], 
                             meta0[dim0]['1d_standard_names']):
            try:
                i = meta1[dim1]['1d_standard_names'].index(sn0)
            except ValueError:
                # Stop: These dimensions do not share the same set of
                #       standard names
                return False
            key1 = meta1[dim1]['1d_coord'][i]

            try:
                equivalent_properties(space0[key0], space1[key1])
            except ValueError:
                # Stop: This dimension in field0 has no matching
                #       dimension in field1 (because space0[key0] has
                #       no matching coordinate in space1)
                return False
            matching_dimensions[matched_dims].append((key0, key1))            
        #--- End: for
    #--- End: for

#- Paired nd auxes
    matching_nd_auxiliaries = []
    for key0, sn0 in (meta0.nd_coords, meta0.nd_standard_names):

        try:
            i = meta1.nd_standard_names.index(sn0)
        except ValueError:
            # Stop: The two fields' Nd coordinates/cell measures do
            #       not share the same set of standard names
            return False
        key1 = meta1.nd_coord[i]

        # Check that potentially matching Nd coordinates/cell measures
        # span equivalent dimensions
        dims0 = space0.dimensions[key0][:]
        dims1 = space1.dimensions[key1][:]
        if len(dims0) != len(dims1):
            # Stop: Nd coordinate/cell measure has no matching
            #       coordinate in the other field
            return False
        for dim0, dim1 in matching_dimensions:
            try:
                dims0.remove(dim0)
            except ValueError:
                pass
            else:
                try:
                    dims1.remove(dim1)
                except ValueError:
                    # Stop: Nd coordinate/cell measure has no matching
                    #       coordinate in the other field
                    return False       
                else:
                    if not dims0:
                         break
        #--- End: for        

        # Check that potentially matching Nd coordinates/cell measures
        # have equivalent properties
        try:
            equivalent_properties(space0[key0], space1[key1])
        except ValueError:
            # Stop: Nd coordinate/cell measure has no matching
            #       coordinate in the other field
            return False

        matching_nd_auxiliaries.append((key0, key1))
    #--- End: for

    # ================================================================
    # There is EXACTLY ONE pair of matching dimensions for which at
    # least one pair of matching 1d coordinates do NOT share the same
    # values for their arrays and bounds. Call this dimension the
    # 'aggregating dimension' in each field.
    # ================================================================
    for matched_dims, matching_coords in matching_dimensions.iteritems():
        for key0, key1 in matching_coords:
            try:
                equal_1d_data(space0[key0], space1[key1])
                # These matching coordinates have equal data
                continue                
            except ValueError:
                # This pair of matching dimensions has a pair of
                # matching coordinates whose data values are not equal
                try:
                    aggregating
                    # Stop: More than one pair of matching dimensions
                    #       have matching coordinates with different
                    #       data values (because name 'aggregating' is
                    #       already defined)
                    return False
                except NameError:
                    # This pair of matching dimensions comprise the
                    # aggregating dimensions, assuming we don't find
                    # another pair with unequal data.
                    aggregating = matched_dims
                    break
        #--- End: for
    #--- End: for

    # ----------------------------------------------------------------
    # Find which pairs of matching coordinates/cell measures (1d and
    # Nd) span their aggregating dimension. Also find, for each field,
    # the key of its aggregating dimension.
    # ----------------------------------------------------------------
    try:        
        spanning_1d_coordinates = matching_dimensions[aggregating]
    except NameError:
        # Stop: There are no aggregating dimensions (because each pair
        #       of matching coordinates has the same data values and
        #       so name 'aggregating' is not defined)
        return False
    adim_key0 = space0.dimensions[spanning_1d_coordinates[0]][0]
    adim_key1 = space1.dimensions[spanning_1d_coordinates[1]][0]

    spanning_nd_auxiliaries     = []
    non_spanning_nd_auxiliaries = []
    for key0, key1 in matching_nd_auxiliaries:
        if adim_key0 in space0.dimensions[key0]:            
            spanning_nd_auxiliaries.append((key0, key1))
        else:
            non_spanning_nd_auxiliaries.append((key0, key1))

    # ----------------------------------------------------------------
    # Check that both or neither aggregating dimensions have a
    # dimension coordinate. (Note that this is stricter than
    # necessary, but means that, for now, we definitely don't have to
    # sort one of the fields along its aggregating dimension.)
    # ----------------------------------------------------------------
    for key0, key1 in spanning_1d_coordinates:
        if key0.startswith('d') and key1.startswith('d'):
            # Found a pair of dimension coordinates
            dim_coords = (key0, key1)
            break

        if key0.startswith('d') or key1.startswith('d'):
            # Stop: Can not (currently) join a dimension coordinate
            #       with an auxiliary coordinate
            return False

    # ================================================================
    # Check that dimension coordinates for the aggregating dimension
    # share no data values
    # ================================================================
    try:
        array1 = space1[dim_coords[0]].varray
        for x in space1[dim_coords[1]].varray.flat:
            if numpy.nonzero(equals(array1, x))[0]:
                # Stop: Dimension coordinates for the aggregating
                #       dimensions share at least one datum
                return False
    except NameError:
        # No dimension coordinates for aggregating dimensions
        # (currently)
        pass
 
    # ----------------------------------------------------------------
    # Check that all field data/coordinates/cell measures which span
    # the aggregating dimension are not already aggregated across a
    # different dimension and that, within each field, they all have
    # the same number of data elements. (Note that we're not checking
    # that each data element has an equivalent aggregating dimension
    # size. I'm assuming that this will be the case. But is this a
    # reasonable assumption?)
    # ----------------------------------------------------------------
    adim_key = adim_key0
    for i, space in emumerate((space0, space1)):
        n_elements = len(field0._data)
    
        # Check spanning 1d coordinates
        for key_pair in spanning_1d_coordinates:
            if len(space[key_pair[i]]._data) != n_elements:
                # Stop: This 1d coordinate has the wrong number of
                #       data elements
                return False                
        #--- End: for

        # Check spanning Ndcoordinates/cell measures
        for key_pair in spanning_nd_auxiliaries:
            key  = key_pair[i]
            if len(space[key]._data) != n_elements:
                # Stop: This Nd coordinate/cell measures has the wrong
                #       number of data elements
                return False
            adim = space.dimensions[key].index(adim_key)
            try:            
                if space[key]._data.adim != adim:
                    # Stop: This Nd coordinate/cell measures is
                    #       already aggregated across a different
                    #       dimension
                    return False
            except AttributeError:
                pass
        #--- End: for

        adim_key   = adim_key1
        n_elements = len(field1._data)
    #--- End: for

    for adim_key, field in zip((adim_key0, adim_key1),
                               (field0   , field1)):
        try:
            adim = field.space.dimensions['data'].index(adim_key)
        except ValueError:
            try:
                field._data.adim
                # Stop: This field is already aggregated across a
                #       different dimension
                return False
            except AttributeError:
                continue
        #--- End: try
        try:            
            if field._data.adim != adim:
                # Stop: This field is already aggregated across a
                #       different dimension
                return False
        except AttributeError:
            pass
    #--- End: for
  
    # ----------------------------------------------------------------
    # Find the insertion indices
    # ----------------------------------------------------------------
    try:
        dim_coords
    except NameError:
        # ------------------------------------------------------------
        # Neither field's aggregating dimension has a dimension
        # coordinate (because name 'dim_coord' is not
        # defined). Therefore the fields will be simply concatenated
        # along the aggregating dimension
        # ------------------------------------------------------------
        indices = len(field0._data)
    else:        
        # ------------------------------------------------------------
        # The aggregating dimensions of both fields have a dimension
        # coordinate. Therefore we need to find the insertion index in
        # field0 for each data element in field1.
        # ------------------------------------------------------------
        mins0 = []
        maxs0 = []
        space0[dim_coords[0]]._data.conform()
        for array in space0[dim_coords[0]]._data._list:
            try:
                first = numpy.asscalar(array)
                mins0.append(first)
                maxs0.append(first)
            except ValueError
                size       = array.size
                first_last = array[slice(0, size, size-1)]
                mins0.append(min(first_last))
                maxs0.append(max(first_last))

        # Set the reversed_ flag to signify whether or not we had to
        # reverse the mins0 and maxs0 lists in order to get them into
        # ascending sequences.
        reversed_ = not space0[dim_coords[0]]._data.direction.values()[0]
        if reversed_             
            mins0.reverse()
            maxs0.reverse()
        
        indices = []
        space1[dim_coords[1]]._data.conform()
        for array in space1[dim_coords[1]]._data._list:
            try:
                first = numpy.asscalar(array)
                min1  = first
                max1  = first
            except ValueError:
                size       = array.size
                first_last = array[slice(0, size, size-1)]
                min1       = min(first_last)
                max1       = max(first_last)

            index = bisect.bisect(mins0, min1)
            
            if reversed_:
                indices.append(len(mins0)-index)
            else:
                indices.append(index)

            # Update mins0 and maxs0
            mins0.insert(index, min1)
            maxs0.insert(index, max1)

            # Check for overlapping dimension coordinates (not allowed
            # for now). This check also serves to check for equal
            # dimension coordinate values.
            try:
                if max1 >= maxs0[index+1]:
                    # Stop: Overlapping or equal dimension coordinate
                    #       values
                    return False
            except IndexError:
                if min0 <= maxs0[index-1]:
                    # Stop: Overlapping or equal dimension coordinate
                    #       values
                    return False
        #--- End: for
    #--- End: if

    # ================================================================
    # Check that no dimension coordinate cell from one field lies
    # entirely within any dimension coordinate cell from the other
    # field. This is a bad, slow method, for now
    # ================================================================
    if dim_coords:
        try:
            bounds0 = space0[dim_coords[0]].bounds.varray
            bounds1 = space1[dim_coords[1]].bounds.varray
        except AttributeError:
            pass
        else:
            for c1 in bounds1:
                min_c1 = min(c1)
                max_c1 = max(c1)
                for c0 in bounds0:
                    min_c0 = min(c0)
                    max_c0 = max(c0)                
                    if (min_c0 <= min_c1 and max_c1 <= max_c0 or
                        min_c1 <= min_c0 and max_c0 <= max_c1):
                        return False

    # ================================================================
    # Check that Nd auxiliary coordinates/cell measures which don't
    # span the aggregating dimension have the same data values
    # ================================================================
    for key0, key1 in non_spanning_nd_auxiliaries:
        try:
            equal_data(space0[key0], space1[key1])
        except ValueError:
            # Stop: Nd auxiliary coordinate/cell measure which does
            #       not span the aggregating dimension has different
            #       values in the two fields
            return False
    #--- End: for

    # ================================================================
    # If either or both aggregating dimensions have a dimension
    # coordinate then there are no values for it and its matching
    # coordinate which appear in both spaces (if they run in different
    # senses or their ranges overlap, the aggregated dimension will
    # need to be sorted). [All other coords are concatenated.]
    # ================================================================


    # ----------------------------------------------------------------
    # Get the variables, from both fields, which span the aggregating
    # dimension ready for data elements from field1 to be inserted
    # into the data object of field0. This basically entails inserting
    # any missing dimensions into the data objects' metadata.
    # ----------------------------------------------------------------


    # ================================================================
    # All of the aggregation tests have been passed and so field1 will
    # now be aggregated into field0 *in place*. Field1 might also be
    # changed *in place*
    # 
    # NOTE: There are no more trapped exceptions from this point
    #       onwards. 
    # ================================================================

    adim_key = adim_key0
    for i, space in emumerate((space0, space1)):
        for key_pair in spanning_1d_coordinates:
            c = space[key_pair[i]]
            try:
                c._data.adim
            except AttributeError:
                
                c._data.adim  = 0                            
                for attr in ('bounds', 'climatology'):
                    try:
                        setattr(getattr(c, attr)._data, adim, 0)
                        break
                    except AttributeError:
                        pass

                if not c._data.order:
                   c._data.shape = (1,)
                   c._data.order = [adim_key]
                   
                   # bounds and direction
                   for attr in ('bounds', 'climatology'):
                       try:
                           b = getattr(c, attr)._data
                       except AttributeError:
                           continue
                       b.shape               = (1, 2)
                       b.order.insert(0, adim_key)
                       b.direction[adim_key] = b.direction['bounds']
                   
                       c._data.direction     = {adim_key: b.direction['bounds']}
                       break
                   #--- End: for
                   if not c._data.direction:
                       # infer the direction from the dimension
                       # coordinate's positive attribute
                       try:
                           if c.positive.lower().startswith('d'): # d for down
                               c._data.direction = False                                
                           else:
                               c._data.direction = True
                       except AttributeError:
                           c._data.direction = True
                #--- End: if
            #--- End: try
        #--- End: for
        adim_key = adim_key1
    #--- End: for

    # Get Nd coordinates and cell measures ready for aggregation
    adim_key = adim_key0
    for i, space in emumerate((space0, space1)):
        for key_pair in spanning_nd_auxiliaries:
            key = key_pair[i]
            c   = space[key]
            try:  
                c._data.adim
            except AttributeError:
                c._data.adim = space.dimensions[key].index(adim_key)
                for attr in ('bounds', 'climatology'):
                    try:
                        setattr(getattr(c, attr)._data, adim, c._data.adim)
                        break
                    except AttributeError:
                        pass
            #--- End: try
        #--- End: for
        adim_key = adim_key1
    #--- End: for

    # Get Data
    for adim_key, field in zip((adim_key0, adim_key1),
                               (field0   , field1   )):
        try:            
            field._data.adim
        except AttributeError:
            try:
                field._data.adim = field.space.dimensions['data'].index(adim_key)
            except ValueError:
                space.dimensions['data'].insert(0, adim_key)

                field._data.adim  = 0
                field._data.shape = (1,) + field._data.shape
                field._data.order.insert(0, adim_key)
                try:
                    direction = field.space[adim_key]._data.direction.values()[0]
                except KeyError:
                    direction = True
                field._data.direction[adim_key] = direction
            #--- End: try
        #--- End: try
    #--- End: for

    # Insert 1d coordinates/cell measures which span the aggregating
    # dimension
    for key0, key1 in spanning_1d_coordinates:
        space0[key0].insert_data(indices, space1[key1], field0, field)
        # Update dimension_sizes
        space0.dimension_sizes[adim_key0] = space0[key0].size

    # Insert Nd coordinates/cell measures which span the aggregating
    # dimension
    for key0, key1 in spanning_nd_auxiliaries:
        space0[key0].insert_data(indices, space1[key1], field0, field1)
 
    # Insert data
    field0.insert_data(indices, field1, field0, field1)
#--- End: def

def equal_data(var0, var1):

    if var0.shape != var1.shape:
        return ValueError, ""

    data_pairs = [(var0, var1)]

    for attr in ('bounds', 'climatology'):
        try:
            bounds0 = getattr(var0, attr)
        except:
            try:
                getattr(var1, attr)
                return ValueError, ""
            except:
                # Neither variable has this bounds
                pass
        else:
            try:
                bounds1 = getattr(var1, attr)
                # Both variables have this bounds
                data_pairs.append((bounds0, bounds1))
                break
            except:
                return ValueError, ""
    #--- End: for

    for data in data_pairs:
        arrays = []
        for var in pair:
            var.conform()
            arrays.append(numpy.concatenate(var._data._list)) # no missing data in coords
        arrays.append(array)
        if not array_allclose(arrays[0], arrays[1]):
            raise ValueError, "Data objects are not equal"
    #--- End: for

    return True
#--- End: def
 

def equal_1d_data(var0, var1):

    size0 = var0.size
    size1 = var1.size

    if size0 != size1:
        return ValueError, ""

    if size0 == 1:
        if not array_allclose(var0.varray, var1.varray):
            raise ValueError, "asdasdasdas76234 -09834 sdf"
    else:
        
        if not array_allclose(extrema0, extrema1):
            raise ValueError, "sdf"
        
        first_last = var0.slice[slice(0, size0, size0-1)].varray
        min0 = numpy.array(min(first_last))
        
        first_last = var1.slice[slice(0, size1, size1-1)].varray
        min1 = numpy.array([min(first_last)])
        
        if not array_allclose(min0, min1):
            raise ValueError, "sdf"
    #--- End: if

    try:
        equal_data(var0, var1)
    except:
        raise ValueError, "asdasdd0000"

    return True
#--- End: def
 
      
#- Each pair of matching coordinates in each pair of dimensions has the
#  same properties and equivalent units.
def equivalent_properties(var0, var1):

    cf_atts = set(var0.cf_atts()) | set(var1.cf_atts())
    cf_atts.pop('standard_name')

    # Check the existence of units
    try:
        var0.units
        var1.units
    except AttributeError:
        raise ValueError, "%s has missing units attribute" % \
            var0.__class__.__name__
    if var0.units != var1.units:
        raise ValueError, "%s have different units" % var0.__class__.__name__
    cf_atts.pop('units')

    # Calendar units
    try:
        cal0 = var0.calendar
    except AttributeError:
        cal0 = 'gregorian'        
    try:
        cal1 = var1.calendar
    except AttributeError:
        cal1 = 'gregorian'
    if cal0 != cal1:
        if not (cal0 in ('standard', 'gregorian') and
                cal1 in ('standard', 'gregorian')):
            raise ValueError, "%s have different calendars" % var0.__class__.__name__
    #--- End: if        
    cf_atts.discard('calendar')

    # Check consistent existence and equality of transform
    attr = 'transform'
    try:
        attr0 = getattr(var0, attr)
    except AttributeError:
        try:
            getattr(var1, attr)
            raise ValueError, "%s doesn't have %s attribute" % \
                (var0.__class__.__name__, attr)
        except AttributeError:
            # Neither variable has a transform attribute 
            pass
    else:
        try:
            attr1 = getattr(var1, attr)
        except AttributeError:
            raise ValueError, "%s doesn't have %s attribute" % \
                (var1.__class__.__name__, attr)
        else:
            try:
                var0.transform['grid_mapping_name']
            except:
                raise ValueError, "Not supporting formula_terms, yet"
            else:
                # Both variables have a transform attribute
                if not cf.equals(attr0, attr1):
                    raise ValueError, "%s doesn't have %s attribute" % \
                        (var0.__class__.__name__, attr)            
    #--- End: try
    cf_atts.discard('transform')

#    # Check consistent existence of bounds or climatology attributes
#    for attr in ('bounds', 'climatology'):
#        try:
#            getattr(var0, attr)
#        except AttributeError:
#            try:
#                getattr(var1, attr)
#                raise ValueError, "%s doesn't have %s attribute" % \
#                    (var0.__class__.__name__, attr)
#            except AttributeError:
#                pass
#        else:
#            try:
#                getattr(var1, attr)
#            except AttributeError:
#                raise ValueError, "%s doesn't have %s attribute" % \
#                    (var1.__class__.__name__, attr)
#        #--- End: try
#    #--- End: for
    cf_atts.discard('bounds')
    cf_atts.discard('climatology')
    
    # Check consistent existence and equality of attributes (we've
    # already checked standard_name)
    for attr in cf_atts:
        try:
            attr0 = getattr(var0, attr)
        except AttributeError:
            try:
                getattr(var1, attr)
                raise ValueError, "%s doesn't have %s attribute" % \
                    (var0.__class__.__name__, attr)
            except AttributeError:
                pass
        else:
            try:
                attr1 = getattr(var1, attr)
            except AttributeError:
                raise ValueError, "%s doesn't have %s attribute" % \
                    (var1.__class__.__name__, attr)
            else:
                if not cf.equals(attr0, attr1):
                    raise ValueError, "%s doesn't have %s attribute" % \
                        (var0.__class__.__name__, attr)
        #--- End: try
    #--- End: for

    return True
#--- End: def                                                            

def equivalent_cell_methods(field0, field1):
    
    try:
        cell_method0 = field0.cell_methods
    except AttributeError:
        try:
            field1.cell_methods
            raise ValueError, "qwerbn37"
        except AttributeError:
            # Neither field has cell methods
            return True
    else:
        try:
            cell_method1 = field1.cell_methods
            # Both fields have cell methods
        except AttributeError:
            raise ValueError, "qwerbn279"

    if len(cell_method0) != len(cell_method1):
        raise ValueError, "qwerbn1743"

    for cm0, cm1 in zip(cell_method0, cell_method1):
        keys0 = set(cm0)
        keys1 = set(cm1)

        if keys0 != keys1:
            raise ValueError, "joof"
        
        for key in keys0 - set(('dim',)):
            if cm0[key] != cm1[key]:
                raise ValueError, "joof"
    #--- End: for

    return True
#--- End: def
