class Units(object):
    '''

A simple, as yet not fully specified, container for CF units and
calendar information.

:keyword **kwargs: The 'units' and 'calendar' attributes may be set
                   with keyword parameters.

**Special attributes:**

================ ==========================
Attribute        Description
================ ==========================
:attr:`calendar` The calendar of time units
:attr:`units`    The units string
================ ==========================

**Examples:**

>>> u = cf.Units(units='days since 2000-1-1', calendar='standard')
>>> u
<CF Units:units='days since 2000-1-1', calendar='standard'>
>>> u.units
'days since 2000-1-1'
>>> u.calendar
'standard'
>>> u == u
True
>>> u.equals(u)
True

**Methods and attributes defined here:**
'''
    
    __slots__ = ['units',
                 'calendar',                
                 ]

    def __init__(self, **kwargs):
        '''

Set the attributes from the arguments.
'''
        for attr, value in kwargs.iteritems():
            setattr(self, attr, value)
    #--- End: def

    def __repr__(self):
        '''
x.__repr__() <==> repr(x)
'''       
        string = ['<CF %s:' % self.__class__.__name__] 
        for attr in self.__slots__:
            try:
                string.append(" %s='%s'" % (attr, getattr(self, attr)))
            except AttributeError:
                pass
        #--- End: for    
        string.append('>')
        return ''.join(string)
    #--- End: def

    def __str__(self):
        '''
x.__str__() <==> str(x)
'''
        try:
            return self.units
        except AttributeError:
            return ''
    #--- End: def

    def __eq__(self, other):
        '''
x.__eq__(y) <==> x==y
'''
        for attr in self.__slots__:
            try:
                value0 = getattr(self, attr)
            except AttributeError:
                try:
                    getattr(other, attr)
                    return False
                except AttributeError:
                    pass
            else:
                try:
                    if value0 != getattr(other, attr):                        
                        return False
                except AttributeError:
                    return False
            #--- End: try
        #--- End: for  

        return True
    #--- End: def

    def __ne__(self, other):
        '''
x.__ne__(y) <==> x!=y
'''
        return not self == other
    #--- End: def

    def __nonzero__(self):
        '''

Returns True if and only the instance has at least one attribute set,
False otherwise.
'''
        for attr in self.__slots__:
            try:
                getattr(self, attr)
                return True
            except AttributeError:
                pass
        #--- End: for

        return False 
    #--- End: def

    def dump(self, id=None, omit=()):
        if id is None:
            id = self.__class__.__name__       
            
        string = []

        for attr in self.__slots__:
            try:
                string.append("%s.%s = '%s'" % (id, attr, getattr(self, attr)))
            except AttributeError:
                pass

        return '\n'.join(string)
    #--- End: def

    def equals(self, other, rtol=None, atol=None):
        '''
Return True if two instances are congruent in that they have

    1. Equal units attributes (if set)
    2. Equal calendar attributes (if set)
    
Equality of numbers is to within a tolerance. Refer to :mod:`cf` for
details.

:argument other: The variable to compare against for equality.

:type other: object
                                                 
:keyword atol: *Optional*. If None then use the default method for
               setting the absolute tolerance for equality of real
               numbers (refer to :mod:`cf` for details). If a float
               then set the absolute tolerance to this value for all
               comparisons (refer to the :obj:`override` parameter).

:type atol: None or float

:keyword rtol: *Optional*. If None then use the default method for
               setting the relative tolerance for equality of real
               numbers (refer to :mod:`cf` for details). If a float
               then set the relative tolerance to this value for all
               comparisons (refer to the :obj:`override` parameter).

:type rtol: None or float


:returns: True if the two objects are congruent, False otherwise.

'''
        return self == other
    #--- End: def

#--- End: class
